import cx_Oracle
# 若出现”cx_Oracle.DatabaseError: ORA-12514: TNS: 监听程序当前无法识别连接描述符中请求的服务“错误，在sql plus中查询下面，然后将service name替换
# select value from v$parameter where name like '%service_name%';

class DalOracle:
    def __init__(self):
        self.connection = False

    def connect(self,connetion_dict: dict):
        try:
            user = connetion_dict['user']
            passwd = connetion_dict['pd']
            hosts = connetion_dict['hosts']
            port = connetion_dict['port']
            dbname = connetion_dict['dbname']
            str_tmp = "%s:%s/%s"%(hosts,port,dbname)
            self.db = cx_Oracle.connect(user, passwd, str_tmp)
            self.mycursor = self.db.cursor()
            self.connection = True
        except Exception as e:
            return str(e)

    def get_all_tables(self):
        """
        获取所有表明
        :return:
        """
        if self.connection:
            try:
                str_sql = "select table_name from user_tables "
                self.mycursor.execute(str_sql)
                sql_res = self.mycursor.fetchall()
                table_list = []
                for each in sql_res:
                    table_list.append(each[0])
                return table_list
            except Exception as e:
                return str(e)
        else:
            return 'please connect to DB'

    def get_table_columns(self, table_name: str):
        """
        获取表所有字段
        :param table_name:
        :return:
        """
        if self.connection:
            try:
                key_type_dict = {}
                str_sql = "SELECT column_name, data_type FROM all_tab_cols WHERE table_name = '%s'" % (table_name)
                self.mycursor.execute(str_sql)
                table_info = self.mycursor.fetchall()
                for each_table_info in table_info:
                    key_type_dict[each_table_info[0]] = each_table_info[1]
                return key_type_dict
            except Exception as e:
                return str(e)
        else:
            return 'please connect to DB'

    def sql_excute(self,sql_e: str):
        """
        执行语句，并返回结果
        :param sql_e:
        :return:
        """
        if self.connection:

            try:
                sql_res = self.mycursor.execute(sql_e)
                ret_res = [each for each in sql_res]
                return ret_res
            except Exception as e:
                return str(e)
        else:
            return 'please connect to DB'
    def get_table_data(self,tableName: str, columnList: list = [] , whereStr: str = '', orderColumn: str = "", Top: int =0):
        """
        获取表数据
        :param tableName:
        :param columnList:
        :param where:
        :return:
        """
        if self.connection:
            try:
                order_str = ''
                where_str = ''
                if not columnList:
                    columnList = []
                    columnList.append('*')

                if whereStr.strip():
                    where_str = 'where ' + whereStr
                    if Top:
                        where_str = where_str + ' and rownum< '+ str(Top)
                elif Top:
                    where_str = 'where rownum< ' + str(Top+1)

                if orderColumn.strip():
                    order_str = "order by " + orderColumn
                sql_e = "select %s from %s %s %s" % (','.join(columnList), tableName, where_str, order_str )

                sql_res = self.mycursor.execute(sql_e)
                ret_res = [each for each in sql_res]
                return ret_res
            except Exception as e:
                return str(e)
        else:
            return 'please connect to DB'
    # def fetch

oracle_connection = DalOracle()
def test():
    conn_dict = {}
    myconn = DalOracle()

    conn_dict['user'] = "system"
    conn_dict['pd'] = "Passw0rd"
    conn_dict['hosts']= "192.168.0.231"
    conn_dict['port']= "1521"
    conn_dict['dbname']= "ZHANG.ZKTEST"

    myconn.connect(conn_dict)
    print(myconn.get_all_tables())
    print(myconn.get_table_columns("LOGMNR_PARAMETER$"))
    print(myconn.get_table_columns("STUDENT"))
    sql_e = 'select * from STUDENT'
    print(myconn.sql_excute(sql_e))

    print(myconn.get_table_data('STUDENT',['XH','XM']))
    print(myconn.get_table_data('STUDENT'))
    print(myconn.get_table_data('STUDENT',['XH','XM'],Top=1))
    print(myconn.get_table_data('STUDENT',Top=1))

if __name__ == '__main__':
    test()





#
# db = cx_Oracle.connect('system','Passw0rd','localhost:1521/ZHANG.ZKTEST')
# print(db.version)
# mycursor = db.cursor()
#
# str_sql = "select table_name from user_tables "
# mycursor.execute(str_sql)
# sql_res = mycursor.fetchall()
# for each in sql_res:
#     print('----------------------%s-----------------------------'%each)
#     str_sql = "SELECT column_name, data_type FROM all_tab_cols WHERE table_name = '%s'"%(each[0])
#     print(str_sql)
#     mycursor.execute(str_sql)
#     table_info = mycursor.fetchall()
#     for each_table_info in table_info:
#         print(each_table_info)



"""
@File    :   DalDecorate.py    
@Software:   PyCharm
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2020/5/15 14:54  kun.z      1.0         None
"""

from functools import wraps
def gen_exception_decorate(obj_func):
    @wraps(obj_func)
    def wrapTheFunction(*args, **kwargs):
        try:
            return obj_func(*args, **kwargs)
        except Exception as e:
            # print(obj_func)
            # print(obj_func.__code__.co_filename)
            err_str = str(obj_func.__code__.co_filename) + str(obj_func.__class__) + str(obj_func.__name__)
            print("程序出错：设计文档 %s 报错，详情如下：%s"%(str(e), err_str))
            return str(e)
    return wrapTheFunction


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import puwrapper_pb2 as puwrapper__pb2


class PURequestStub(object):
  """The pu request service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Request = channel.unary_unary(
        '/puwrapper.PURequest/Request',
        request_serializer=puwrapper__pb2.RequestMsg.SerializeToString,
        response_deserializer=puwrapper__pb2.RequestRetMsg.FromString,
        )


class PURequestServicer(object):
  """The pu request service definition.
  """

  def Request(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_PURequestServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Request': grpc.unary_unary_rpc_method_handler(
          servicer.Request,
          request_deserializer=puwrapper__pb2.RequestMsg.FromString,
          response_serializer=puwrapper__pb2.RequestRetMsg.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'puwrapper.PURequest', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import os
import click
import json

runscript = '''
import click
from concurrent import futures
import grpc
import puwrapper_pb2
import puwrapper_pb2_grpc
from %s import %s

@click.command()
@click.option('--wport', default=0, help='java wrapper grpc server port')
@click.option('--port', default=0, help='python grpc server port')
def serve(wport,port):
    if wport == 0:
        print('Wport should be set!')
        return
    if port == 0:
        print('Port should be set!')
        return
    pcu = %s() 
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    puwrapper_pb2_grpc.add_PURequestServicer_to_server(
        %s(), server)
    server.add_insecure_port("[::]:{}".format(port))
    server.start()
    server.wait_for_termination()

if __name__ == '__main__':
    serve()
'''

@click.command()
@click.option('--folder', help='target folder')
def run(folder):
    #read in unit info
    dicInfo={}
    if folder is None:
        print('Folder should be set, use --help see help!')
        return
    infoF = folder+'\\info.json'
    file_info = open(infoF, 'r')
    # content = file_info.read()    
    # print(file_info)
    dicInfo = json.load(file_info)
    file_info.close()    
    if 'name' not in dicInfo:
        print('Info.json format wrong, no name field found!')
        return    
    #write runscript
    name = dicInfo['name']
    scriptF = folder+'\\run.py'
    file_script = open(scriptF, 'w')
    file_script.write(runscript%(name,name,name,name))
    file_script.close( )


if __name__ == '__main__':
    run()



import re
import time
import json
import inspect
import threading
from .puwrapper_pb2_grpc import PURequestServicer
from .puwrapper_pb2 import RequestRetMsg

class DALJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return str(obj, encoding='utf-8')
        return json.JSONEncoder.default(self, obj)


class DalBaseUnit(PURequestServicer):

    inprocess = False
    dataIF = []
    threads = []
    dicMethod = {}

    def _start_frame(self):
        difIF = {}
        if self.inprocess == True:
            print('Already in routine, do not call this function twice!')
            return
        funcs = inspect.getmembers(self, predicate=inspect.ismethod)
        self.inprocess = True
        for (name, method) in funcs:
            self.dicMethod[name] = method
            matchObj = re.match('(.*)_Info',name)
            if matchObj is not None:
                difIF['fidx'] = matchObj.group(1)
                difIF['info'] = method()
                self.dataIF.append(difIF)
            if re.match('(.*)_NewData',name) is not None:
                p = threading.Thread(target=DalBaseUnit._run_new_data,args=(self,method))
                p.start()
                self.threads.append(p)
    
    def _run_new_data(self,fun_new_data):
        while self.inprocess:
            fun_new_data()
            time.sleep(0.01)    

    def logError(self,content):
        return

    def logWarn(self,content):
        return

    def logInfo(self,content):
        return

    def quit(self):
        self.inprocess = False
        for th in self.threads:
            th.join()
        
    def newdata(self,dicMeta,dicData):
        print('New data meta[{}], data[{}]'.format(dicMeta,dicData))

    def Request(self, request, context):
        retDic = {}
        funName = ''
        CT = True
        dictReq = json.loads(request.strReq)

        retDic['retCode'] = 0
        retDic['retInfo'] = 'Success'
        retDic['retContent'] = ''
        dictUser = dictReq['user']
        dictInput = dictReq['input'] 
        if dictInput['type'] == 'QuerySystem':
            retDic['retContent'] = json.dumps(self.dataIF)
            CT = False
        if CT and ('user' not in dictReq or 'input' not in dictReq):
            retDic['retCode'] = 1
            retDic['retInfo'] = 'Input parameter format wrong, user and input should be set!'
            CT = False         
        if CT and ('fidx' not in dictInput or 'type' not in dictInput):
            retDic['retCode'] = 1
            retDic['retInfo'] = 'Fidx or type not defined'
            CT = False 
        if CT:
            if dictInput['type'] == 'QueryMeta':
                funName = dictInput['fidx']+'_Meta'
            elif dictInput['type'] == 'QueryData':
                funName = dictInput['fidx']+'_Data'
            elif dictInput['type'] == 'SubMeta':
                #todo
                todo = 1
            elif dictInput['type'] == 'SubData': 
                #todo
                todo = 1
            else:
                retDic['retCode'] = 3
                retDic['retInfo'] = 'Operation type should be one of QueryMeta,QueryData,SubMeta,SubData'
                CT = False  
        if CT and (funName not in self.dicMethod):
            retDic['retCode'] = 4
            retDic['retInfo'] = 'Function {} not fount in this unit!'.format(funName)
            CT = False            
        if CT:
            retDic = self.dicMethod[funName](dictUser,dictInput)
        ret = RequestRetMsg(strRet=json.dumps(retDic,cls=DALJsonEncoder))
        return ret


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: puwrapper.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='puwrapper.proto',
  package='puwrapper',
  syntax='proto3',
  serialized_options=_b('\n\024com.daline.puwrapperB\014RequestProtoP\001'),
  serialized_pb=_b('\n\x0fpuwrapper.proto\x12\tpuwrapper\"\x1c\n\nRequestMsg\x12\x0e\n\x06strReq\x18\x01 \x01(\t\"\x1f\n\rRequestRetMsg\x12\x0e\n\x06strRet\x18\x01 \x01(\t2I\n\tPURequest\x12<\n\x07Request\x12\x15.puwrapper.RequestMsg\x1a\x18.puwrapper.RequestRetMsg\"\x00\x42&\n\x14\x63om.daline.puwrapperB\x0cRequestProtoP\x01\x62\x06proto3')
)




_REQUESTMSG = _descriptor.Descriptor(
  name='RequestMsg',
  full_name='puwrapper.RequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strReq', full_name='puwrapper.RequestMsg.strReq', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30,
  serialized_end=58,
)


_REQUESTRETMSG = _descriptor.Descriptor(
  name='RequestRetMsg',
  full_name='puwrapper.RequestRetMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strRet', full_name='puwrapper.RequestRetMsg.strRet', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=91,
)

DESCRIPTOR.message_types_by_name['RequestMsg'] = _REQUESTMSG
DESCRIPTOR.message_types_by_name['RequestRetMsg'] = _REQUESTRETMSG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RequestMsg = _reflection.GeneratedProtocolMessageType('RequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTMSG,
  '__module__' : 'puwrapper_pb2'
  # @@protoc_insertion_point(class_scope:puwrapper.RequestMsg)
  })
_sym_db.RegisterMessage(RequestMsg)

RequestRetMsg = _reflection.GeneratedProtocolMessageType('RequestRetMsg', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTRETMSG,
  '__module__' : 'puwrapper_pb2'
  # @@protoc_insertion_point(class_scope:puwrapper.RequestRetMsg)
  })
_sym_db.RegisterMessage(RequestRetMsg)


DESCRIPTOR._options = None

_PUREQUEST = _descriptor.ServiceDescriptor(
  name='PURequest',
  full_name='puwrapper.PURequest',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=93,
  serialized_end=166,
  methods=[
  _descriptor.MethodDescriptor(
    name='Request',
    full_name='puwrapper.PURequest.Request',
    index=0,
    containing_service=None,
    input_type=_REQUESTMSG,
    output_type=_REQUESTRETMSG,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_PUREQUEST)

DESCRIPTOR.services_by_name['PURequest'] = _PUREQUEST

# @@protoc_insertion_point(module_scope)



import os
import json
from .Template import DIC_TEMPLATE
from .DesignFile import DesignFile
from .GenTestData import gem_test_data
from .DalDecorate import gen_exception_decorate
from .OracleOper import oracle_connection

class DalGen:

    def __init__(self):
        super().__init__()
        self.template_list = DIC_TEMPLATE

    def _get_config(self, config_file):
        """
        读取配置文件，比如：包括模板文件位置等
        :param config_file:
        :return:
        """
        pass

    def _get_all_template_type(self):
        """
        获取所有模板类型
        :return:所有类型
        """
        return self.template_list.keys()

    def read_design_file(self, design_file_name: str):
        """
        读取设计文件的文档，返回设计文件的关键字
        :param design_file_name:设计文件文档
        :return:文档关键字字典，不做成员变量，返回格式{'sttime':'datetime','symbol':'string'}
        """
        self._design_file = DesignFile(design_file_name)
        return self._design_file.get_all_design_unit()

    def _gen_test_data(self, data_dic: dict):
        """
        给定数据类型，生成测试数据
        Enum：列表中随机选取1个数据
        基本数据类型: 按类型创建

        :param data_dic: 单个单元的设计字典，格式{'sttime':'datetime','symbol':'string'}
        :return:测试数据 json
        """
        for each_val in data_dic:
            if isinstance(data_dic[each_val],dict):
                data_dic[each_val] = self._gen_test_data(data_dic[each_val])  # 递归实现无限遍历
            else:
                data_dic[each_val] = gem_test_data.gen_test_data_str(data_dic[each_val])
        return data_dic
        # return json.dumps(data_dic)

    def _gen_get_upper_units_data(self, unit_name_list: list):
        """
        生成获取父节点数据的方法
        1、若能找到父节点的文档或单元信息，将父节点的paralist作为参数填入
        2、找不到父节点，生成空函数
        :param unit_name_list: 上级unit名称列表
        :return:父节点的数据
        """
        ret_scritp = ''
        all_unit_dic = self._design_file.get_all_design_unit()  # todo: 目前无法获取全部的unit单元函数信息，暂用设计文档的信息替代
        for each_parent_unit in unit_name_list:
            if each_parent_unit in all_unit_dic:
                func_script_tmp = ''
                for each_upper_function in all_unit_dic[each_parent_unit]['function_list']:
                    scritpTemplate = DIC_TEMPLATE['ST_PU']['get_upper_unit_data_template']
                    scritpTemplate = scritpTemplate.replace('$UNIT_NAME$',all_unit_dic[each_parent_unit]['unit_info']['unitname'])
                    scritpTemplate = scritpTemplate.replace('$FUNCTION_NAME$',each_upper_function)
                    str_tmp = 'self,' + ','.join( all_unit_dic[each_parent_unit]['function_list'][each_upper_function]['input'].keys() )
                    scritpTemplate = scritpTemplate.replace('$UPPER_INPUTE$',str_tmp)
                    str_tmp = json.dumps( self._gen_test_data(all_unit_dic[each_parent_unit]['function_list'][each_upper_function]['output']) )
                    scritpTemplate = scritpTemplate.replace('$UPPER_OUTPUT_EXAMPLE$',str_tmp)
                    func_script_tmp = func_script_tmp + scritpTemplate
            else:
                func_script_tmp = ''
            ret_scritp = ret_scritp + func_script_tmp
        return ret_scritp

    def _gen_unit_function_script(self,function_dict: dict):
        """
        根据函数字典生成函数脚本
        :param function_dict:
        :return:
        """
        ret_script = ''
        for each_func in function_dict:
            tmp_script = ''
            tmp_script = tmp_script + DIC_TEMPLATE['ST_PU']['function_method']
            tmp_script = tmp_script.replace('$FUNCTION_NAME$',each_func)
            tmp_script = tmp_script.replace('$AC_CODE$',function_dict[each_func]['meta_data']['acCode'])
            tmp_script = tmp_script.replace('$DESCRIPTION$',function_dict[each_func]['meta_data']['description'])
            tmp_script = tmp_script.replace('$INPUTE_EXAMPLE$',json.dumps(function_dict[each_func]['input']) )
            tmp_script = tmp_script.replace('$META_DATA$',json.dumps(function_dict[each_func]['meta_data'],ensure_ascii=False) )
            tmp_script = tmp_script.replace('$RET_DATA$',json.dumps(self._gen_test_data(function_dict[each_func]['output']) ) )

            ret_script = ret_script + tmp_script
        return ret_script

    def _gen_test_script(self,function_dict: dict):
        """
        根据函数字典生成函数测试脚本
        :param function_dict:
        :return:
        """
        tmp_script = DIC_TEMPLATE['ST_PU']['test_template']
        tmp_func_script = ''
        for each_func in function_dict:
            # 对每个函数生成 info  meta   data三个测试用例函数
            str_tmp = DIC_TEMPLATE['ST_PU']['test_functest_template']
            str_tmp = str_tmp.replace('$FUNCTION_NAME$',each_func)
            str_tmp = str_tmp.replace('$INPUTE_EXAMPLE$',json.dumps(function_dict[each_func]['input']) )
            tmp_func_script = tmp_func_script + str_tmp
        tmp_script = tmp_script.replace('$FUNCTION_TEST$',tmp_func_script)

        return tmp_script
    def _gen_Oracle_script(self, DB_dict: dict):
        """
        生成数据库脚本
        :param DB_dict:
        :return:
        """
        db_script_dic = {}

        # 初始化模板
        db_script_dic['db_init'] = DIC_TEMPLATE['ST_PU']['ORACLE_INIT_Template']
        db_script_dic['db_init'] = db_script_dic['db_init'].replace("$USER$",DB_dict['user'])
        db_script_dic['db_init'] = db_script_dic['db_init'].replace("$PD$",DB_dict['pd'])
        db_script_dic['db_init'] = db_script_dic['db_init'].replace("$HOST$",DB_dict['hosts'])
        db_script_dic['db_init'] = db_script_dic['db_init'].replace("$PORT$",DB_dict['port'])
        db_script_dic['db_init'] = db_script_dic['db_init'].replace("$DBNAME$",DB_dict['dbname'])

        oracle_connection.connect(DB_dict)
        if not oracle_connection.connection:
            db_script_dic['method_script'] = ''
            return db_script_dic

        # 表函数模板
        db_script_dic['method_script'] = ''

        tablelist = oracle_connection.get_all_tables()
        for each_table in tablelist:
            each_table_tmp = each_table
            if "$" in each_table_tmp:
                each_table_tmp = each_table_tmp.replace('$','')
            table_script_tmp = DIC_TEMPLATE['ST_PU']['ORACLE_TALBE_Template']
            table_script_tmp = table_script_tmp.replace('$TABLENAME$',each_table_tmp)
            db_script_dic['method_script'] = db_script_dic['method_script']+table_script_tmp
        if 'SQL' in DB_dict:
            for each_sql_name in DB_dict['SQL']:
                sql_script_tmp = DIC_TEMPLATE['ST_PU']['ORACLE_SQL_Template']
                sql_script_tmp = sql_script_tmp.replace('$SQLNAME$',each_sql_name)
                sql_script_tmp = sql_script_tmp.replace('$SQLSTRING$',DB_dict['SQL'][each_sql_name])
                db_script_dic['method_script'] = db_script_dic['method_script'] + sql_script_tmp
        return db_script_dic
    def _gen_SqlServer_script(self, DB_dict: dict):
        """
        生成数据库脚本
        :param DB_dict:
        :return:
        """
        db_script_dic = {}
        db_script_dic['method_script'] = ''
        db_script_dic['db_init'] = ''

    def _gen_Mysql_script(self, DB_dict: dict):
        """
        生成数据库脚本
        :param DB_dict:
        :return:
        """
        db_script_dic = {}
        db_script_dic['method_script'] = ''
        db_script_dic['db_init'] = ''


    def _gen_DB_script(self, DB_dict: dict):
        """
        生成DB部分的脚本,需要连接数据库，然后扫描所有表
        需要生成连接函数，在init中调用
        设计文档可能连不了数据库，无法得到表结构，则打印提示，生成注释的示例
        :param DB_dict:
        :return:
        """
        if DB_dict['dbtype'].lower() == 'oracle':
            db_script_dic = self._gen_Oracle_script(DB_dict)
        elif DB_dict['dbtype'].lower() == 'sqlserver':
            db_script_dic = self._gen_SqlServer_script(DB_dict)
        elif DB_dict['dbtype'].lower() == 'mysql':
            db_script_dic = self._gen_Mysql_script(DB_dict)

        return db_script_dic

    @gen_exception_decorate
    def gen_py(self, unit_dic: dict):
        """
        生成py文件
        :param unit_dic: 单个单元的设计字典
        :param gen_test_data: 是否生成测试数据
        :return:
        """
        if unit_dic['unit_info']['type'] == 'pu':
            scritpTemplate = DIC_TEMPLATE['ST_PU']['frame']
            # unit名称替换
            scritpTemplate =scritpTemplate.replace('$CLASS_NAME$',unit_dic['unit_info']['unitname'])

            # 将每个方法生成四个函数
            str_tmp = self._gen_unit_function_script(unit_dic['function_list'])  # done
            scritpTemplate =scritpTemplate.replace('$FUNCTION_METHOD$',str_tmp)

            if 'data_base' in unit_dic and unit_dic['data_base']:
                # 根据数据库配置生成function，每个表生成简单的获取表数据语句，此方法不同于function，只生成单个的函数
                db_script_dic = self._gen_DB_script(unit_dic['data_base'])
                scritpTemplate = scritpTemplate.replace('$DB_METHOD$', db_script_dic['method_script'])
                scritpTemplate = scritpTemplate.replace('$DB_INIT$', db_script_dic['db_init'])
            else:
                scritpTemplate = scritpTemplate.replace('$DB_METHOD$', '')
                scritpTemplate = scritpTemplate.replace('$DB_INIT$', '')

            # 调用上层pu方法脚本生成，如果db_unit不会同时支持upper_unit，将下部分移到上面else后面
            if 'upper_unit' in unit_dic:
                str_tmp = self._gen_get_upper_units_data(unit_dic['upper_unit'])  # done
                scritpTemplate =scritpTemplate.replace('$CALL_UPPER_UNITS$',str_tmp)
            else:
                scritpTemplate = scritpTemplate.replace('$CALL_UPPER_UNITS$', '')

            # 测试用例生成
            str_tmp = self._gen_test_script(unit_dic['function_list'])  # done
            scritpTemplate =scritpTemplate.replace('$TEST$', str_tmp )

            # 写文件
            if not os.path.exists(unit_dic['unit_info']['folder']):
                os.mkdir(unit_dic['unit_info']['folder'])
            sub_path = unit_dic['unit_info']['folder']+unit_dic['unit_info']['unitname']+'\\'
            if not os.path.exists(sub_path):
                os.mkdir(sub_path)
            scriptF = sub_path + unit_dic['unit_info']['unitname'] + '.py'
            file_script = open(scriptF, 'w+',encoding='utf8')
            file_script.write(scritpTemplate )
            file_script.close()
        elif unit_dic['unit_info']['type'] == 'tu':
            scritpTemplate = DIC_TEMPLATE['ST_TU']
        else:
            # print('type should be pu or tu')
            return 'type should be pu or tu'

    @gen_exception_decorate
    def gen_run(self, unit_dic: dict):
        """
        生成运行测试文件
        """
        run_template = DIC_TEMPLATE['ST_PU']['run_template']
        run_template = run_template.replace('$CLASS_NAME$',unit_dic['unit_info']['unitname'])
        # 写文件
        if not os.path.exists(unit_dic['unit_info']['folder']):
            os.mkdir(unit_dic['unit_info']['folder'])
        sub_path = unit_dic['unit_info']['folder'] + unit_dic['unit_info']['unitname'] + '\\'
        if not os.path.exists(sub_path):
            os.mkdir(sub_path)
        scriptF = sub_path + 'run_test.py'
        file_script = open(scriptF, 'w+', encoding='utf8')
        file_script.write(run_template)
        file_script.close()
        pass

    @gen_exception_decorate
    def gen_info(self, unit_dic: dict):
        """
        生成info文件
        :param unit_dic: 单个单元的设计字典
        :return:
        """
        dicInfo = {}
        dicInfo['folder'] = unit_dic['unit_info']['folder']
        dicInfo['name'] = unit_dic['unit_info']['unitname']
        dicInfo['type'] = unit_dic['unit_info']['type']
        dicInfo['lang'] = unit_dic['unit_info']['lang']

        if not os.path.exists(unit_dic['unit_info']['folder']):
            os.mkdir(unit_dic['unit_info']['folder'])
        sub_path = unit_dic['unit_info']['folder'] + unit_dic['unit_info']['unitname'] + '\\'
        if not os.path.exists(sub_path):
            os.mkdir(sub_path)
        infoF = sub_path +unit_dic['unit_info']['unitname']+ '_info.json'
        file_script = open(infoF, 'w+',encoding='utf8')
        file_script.write(json.dumps(dicInfo))
        file_script.close()

    def gen_unit(self, design_file: str):
        """
        通过设计文档生成unit单元
        :param design_file: 设计文档
        :param gen_test_data: 是否生成测试数据
        :return:
        """
        design_units = self.read_design_file(design_file)
        self._design_file.gen_design_md()
        for each_unit in design_units:
            self.gen_py(design_units[each_unit])
            self.gen_info(design_units[each_unit])
            self.gen_run(design_units[each_unit])
import click
@click.command()
@click.option('--designPath', help='target design file path')
def run(designPath):
    dalgen = DalGen()
    dalgen.gen_unit(designPath)

def GenScriptFromMd(designPath):
    dalgen = DalGen()
    dalgen.gen_unit(designPath)

if __name__ == '__main__':
    # md_path = input('请输入设计文档路径：')
    # GenScriptFromMd(md_path)
    GenScriptFromMd(r'UnitDesign\design_test2.md')
    # GenScriptFromMd(r'UnitDesign\design.md')



"""
@File    :   GenTestData.py    
@Contact :   zhangkun0518@outlook.com
@Software:   PyCharm
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2020/5/12 17:47  kun.z      1.0         None
"""
import re
import datetime
import random
import string
class GenTestData():
    """
    随机生成字符串
    """
    def gen_test_data_str(self,data_type: str):
        data_type_lit = data_type.split('+')
        str_ret = ''
        p1 = re.compile(r'[(](.*?)[)]', re.S)
        for each in data_type_lit:
            if 'datetime' in each:
                tmp = re.findall(p1, each)
                day_random = random.randint(1,100)
                second_random = random.randint(1,10000)
                if not tmp:
                    tmp = ['%Y-%m-%d %H:%M:%S']
                str_ret = str_ret + (datetime.datetime.now() - datetime.timedelta(days=day_random,seconds=second_random)).strftime(tmp[0])
            elif 'int' in each:
                data_len = 5
                tmp = re.findall(p1, each)
                if tmp:
                    data_len = int(tmp[0])
                num_int = random.randint(pow(10,(data_len-1)),pow(10,(data_len-0)) )
                str_ret = str_ret + str( num_int )

            elif 'float' in each:
                int_len = 5
                decimal_len = 5
                tmp = re.findall(p1, each)
                if tmp:
                    tmp_list = tmp[0].split('.')
                    int_len = int(tmp_list[0])
                    if len(tmp_list)>1:
                        decimal_len = int(tmp_list[0])
                num_int = random.randint(pow(10,(int_len-1)),pow(10,(int_len-0)) )
                num_decimal = random.randint(pow(10,(decimal_len-1)),pow(10,(decimal_len-0)) )
                str_ret = str_ret + str( num_int ) + '.' + str(num_decimal)
            elif 'str' in each:
                data_len = 5
                tmp = re.findall(p1, each)
                if tmp:
                    data_len = int(tmp[0])
                for i in range(0,data_len):
                   str_ret = str_ret + random.choice(string.ascii_letters)
    
        return str_ret
        
gem_test_data = GenTestData()
def mytest():
    tmp = GenTestData()
    data1 = 'int(8)'
    print(tmp.gen_test_data_str(data1))
    data1 = 'float(4.4)'
    print(tmp.gen_test_data_str(data1))
    data1 = 'str(8)'
    print(tmp.gen_test_data_str(data1))
    data1 = 'datetime(%Y-%m-%d %H:%M:%S)'
    print(tmp.gen_test_data_str(data1))
    data1 = 'str(3)+int(4)'
    print(tmp.gen_test_data_str(data1))
if __name__=='__main__':
    mytest()


"""
@File    :   Template.py    
@Software:   PyCharm
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2020/5/11 18:41  kun.z      1.0         None
"""
DIC_TEMPLATE = {} # unit单元的模板文档
# DIC_TEMPLATE_GET_UNIT_DATA = {} # 获取unit数据的函数模板
DIC_TEMPLATE['ST_PU'] = {}

DIC_TEMPLATE['ST_PU']['frame'] = """
import time
from DalBaseUnit import DalBaseUnit

class $CLASS_NAME$(DalBaseUnit):
    def __init__(self):
        self._start_frame()
        $DB_INIT$
    $FUNCTION_METHOD$
    $DB_METHOD$
    $CALL_UPPER_UNITS$
    $TEST$
if __name__ == '__main__':
    pcu = $CLASS_NAME$()
    pcu.test()
"""

DIC_TEMPLATE['ST_PU']['function_method'] ="""
    def $FUNCTION_NAME$_Info(self):
        # 函数信息
        retInfo={}
        retInfo['accode'] = '$AC_CODE$'
        retInfo['desc'] = '$DESCRIPTION$'
        # 输入的示例
        retInfo['example'] = \'\'\'
            $INPUTE_EXAMPLE$
        \'\'\'
        return retInfo

    def $FUNCTION_NAME$_Meta(self,dicUser,dicInput):
        # 数据权限、版本、时间、元数据大小、压缩数据大小
        retMeta='$META_DATA$'
        return retMeta

    def $FUNCTION_NAME$_Data(self,dicUser,tupleMeta):
        # 请将输出数据作为retData返回
        retData={}        
        # ----------测试数据------------
        retData = $RET_DATA$
        return retData

    def $FUNCTION_NAME$_NewData(self):
        # 数据订阅的更新入口
        dicMeta=$META_DATA$
        dicData=$RET_DATA$
        self.newdata(dicMeta,dicData)
        time.sleep(2)
"""

DIC_TEMPLATE['ST_PU']['get_upper_unit_data_template'] = """
    def _get_$UNIT_NAME$_$FUNCTION_NAME$_data($UPPER_INPUTE$):
        ret_data = {}
        # ----------测试数据------------
        ret_data = $UPPER_OUTPUT_EXAMPLE$
        return ret_data
"""

DIC_TEMPLATE['ST_PU']['test_template'] ="""
    def test(self):
        dicUser={}
        dicUser['name']='hellodal'
        dicUser['accountStauts']='normal'
        dicUser['uidNumber']=500
        dicUser['gidNumber']=200
        dicUser['description']='this is a sample user'
        dicUser['listGPID']=['201','202']
        $FUNCTION_TEST$
        time.sleep(20)
        self.quit()
"""

DIC_TEMPLATE['ST_PU']['test_functest_template'] = """
        dicInfo = self.$FUNCTION_NAME$_Info()
        print('$FUNCTION_NAME$ info:{}',dicInfo)
        dicInput=$INPUTE_EXAMPLE$
        dicMeta = self.$FUNCTION_NAME$_Meta(dicUser,dicInput)
        print('$FUNCTION_NAME$ meta:{}',dicMeta)
        dicData = self.$FUNCTION_NAME$_Data(dicUser,dicMeta)
        print('$FUNCTION_NAME$ data:{}',dicData)
"""

DIC_TEMPLATE['ST_PU']['run_template'] = """
from concurrent import futures
import grpc
import puwrapper_pb2_grpc
from $CLASS_NAME$ import $CLASS_NAME$

def serve(wport,port):
    if wport == 0:
        print('Wport should be set!')
        return
    if port == 0:
        print('Port should be set!')
        return
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    puwrapper_pb2_grpc.add_PURequestServicer_to_server(
        $CLASS_NAME$(), server)
    server.add_insecure_port("[::]:{}".format(port))
    server.start()
    print('----------unit started---------')
    server.wait_for_termination()

if __name__ == '__main__':
    serve(9091,9092)
"""

DIC_TEMPLATE['ST_PU']['ORACLE_TALBE_Template'] = """
    def _get_$TABLENAME$_data(self):
        column_list = self.myconn.get_table_columns("$TABLENAME$")
        columns_to_get = list(column_list.keys())
        res_ret = self.myconn.get_table_data("$TABLENAME$", columnList=columns_to_get,
                                             whereStr='', orderColumn='', Top=10)
        return res_ret
"""
DIC_TEMPLATE['ST_PU']['ORACLE_SQL_Template'] = """
    def _execute_$SQLNAME$_data(self):
        res_ret = self.myconn.sql_excute('$SQLSTRING$')
        return res_ret
"""

DIC_TEMPLATE['ST_PU']['ORACLE_INIT_Template'] = """
        from OracleOper import DalOracle
        self.myconn = DalOracle()
        conn_dict = {}
        conn_dict['user'] = "$USER$"
        conn_dict['pd'] = "$PD$"
        conn_dict['hosts']= "$HOST$"
        conn_dict['port']= "$PORT$"
        conn_dict['dbname']= "$DBNAME$"
        self.myconn.connect(conn_dict)
"""


DIC_TEMPLATE['ST_TU'] = {}
DIC_TEMPLATE['ST_TU']['frame']="""
import os
%s
%s

if __name__ == '__main__'
    run()
"""


# print(DIC_TEMPLATE['ST_PU'].keys())


"""
@File    :   DesignFile.py
@Software:   PyCharm
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2020/5/12 9:45  kun.z      1.0         None
"""
import re
import json
from .DalDecorate import gen_exception_decorate

MD_GRAPH_TEMPLATE = """
```mermaid
    graph TB
$GRAPH_CODE$
```
        """

class DesignFile():
    """
    设计文档类，用来获取设计文档的内容
    """
    def __init__(self,filename = ''):
        self._dic_unit = {}
        if filename:
            self.read_design_file(filename)
    def read_design_file(self, filename):
        """
        读取设计文档
        :param filename:设计文档名称
        :return:设计文档各单元
        """
        self._file_name = filename
        with open(filename,encoding='utf8') as f:
            unit_name = ''
            second_title_name = ''
            # 只有function_list有三四级标题
            third_title_name = ''
            fourth_title_name = ''
            out_json_str = ''
            for line in f:
                if re.match('>',line.strip()) or re.match('//',line.strip()) or re.match('---',line.strip()) :
                    # md 的注释文档、分割线  跳过
                    continue
                if not line.strip():
                    # 空行跳过
                    continue
                if re.match('# ',line):
                    unit_name = line.strip('# ').strip()
                    self._dic_unit[unit_name] = {}
                    self._dic_unit[unit_name]['unit_info'] = {}
                    self._dic_unit[unit_name]['unit_info']['unitname'] = unit_name
                    second_title_name = ''
                    third_title_name = ''
                    fourth_title_name = ''
                    continue
                # --------二级标题切换-----------
                elif re.match('## ',line):
                    second_title_name = line.strip('## ').strip()
                    if 'unit_info' not in line: # 上面已经定义了unit_info,并赋值了，其余的都可以初始化
                        self._dic_unit[unit_name][second_title_name] = {}
                        third_title_name = ''
                        fourth_title_name = ''
                        continue
                # --------三级标题切换-----------
                elif re.match('### ',line):
                    third_title_name = line.strip('### ').strip().strip('()')
                    self._dic_unit[unit_name][second_title_name][third_title_name] = {}
                    fourth_title_name = ''
                    continue
                # --------四级标题切换-----------
                # 只有function_list有四级标题
                elif re.match('#### ',line):
                    fourth_title_name = line.strip('#### ').strip()
                    self._dic_unit[unit_name][second_title_name][third_title_name][fourth_title_name] = {}
                    out_json_str = ''
                    continue

                elif '+' in line:
                    line_tmp = line.strip('+').strip(' ').split("#")[0].split('=')
                    if len(line_tmp) == 1:
                        line_tmp.append('')
                    if third_title_name :
                        if fourth_title_name :
                            self._dic_unit[unit_name][second_title_name][third_title_name][fourth_title_name][
                                line_tmp[0].strip()] = line_tmp[1].strip()
                        else:
                            self._dic_unit[unit_name][second_title_name][third_title_name][line_tmp[0].strip()] = \
                            line_tmp[1].strip()
                    else:
                        self._dic_unit[unit_name][second_title_name][line_tmp[0].strip()] = line_tmp[1].strip()
                    continue

                # --------内容读取，output,function_list-----------
                # output的数据开头没有标志，仅仅用四级标题确定
                elif 'output' in fourth_title_name :
                    out_json_str = out_json_str + line.split('#')[0]
                    try:
                        self._dic_unit[unit_name][second_title_name][third_title_name][fourth_title_name] = json.loads(out_json_str)
                    except Exception as e:
                        pass
                    continue
            # print(out_json_str)
        return self._dic_unit
    def get_all_design_unit(self):
        """
        获取所有设计的单元
        :return:所有设计单元
        """
        return self._dic_unit

    @gen_exception_decorate
    def gen_design_md(self):
        """
        重新生成设计文档的md文档，包括mermaid的图
        :return:
        """
        f_old = open(self._file_name,'r',encoding='utf8')
        old_context = f_old.readlines()
        file_name_gen = '%s.%s'%(self._file_name.split('.')[0]+'_gen',self._file_name.split('.')[1])
        f_gen = open(file_name_gen,'w',encoding='utf8')

        script_graph = ''
        for each_unit in self._dic_unit:
            if 'upper_unit' not in self._dic_unit[each_unit]:
                continue
            for each_data in self._dic_unit[each_unit]['upper_unit']:
                script_graph = script_graph + '\t\t%s[%s]-->%s[%s]\n'%(each_data,each_data,self._dic_unit[each_unit]['unit_info']['unitname'],self._dic_unit[each_unit]['unit_info']['unitname'])
        script_graph = MD_GRAPH_TEMPLATE.replace('$GRAPH_CODE$', script_graph)
        f_gen.writelines(old_context)
        f_gen.write('----\n')
        f_gen.write('# 拓扑图如下')
        f_gen.write(script_graph)

        f_gen.close()
        f_old.close()
    def gen_design_PDF(self):
        """
        生成设计文档的PDF
        :return:
        """
        return '暂不支持'

def mytest():
    tmp = DesignFile()
    print('read_design_file:'+str(tmp.read_design_file(r'UnitDesign\design.md')))
    print("get_all_design_unit:"+str(tmp.get_all_design_unit()))

    tmp2 = DesignFile(r'UnitDesign\design.md')
    print("init函数读文件get_all_design_unit:"+str(tmp2.get_all_design_unit()))
    tmp2.gen_design_md()
if __name__=='__main__':
    mytest()


import setuptools

with open("readme.md", "r",encoding='utf8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="DalineUnit",
    version="0.0.14",
    author="kun.z",
    author_email="kun.z@daline.com.cn",
    description="daline unit generator and runner",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DalineWH",
    packages=['DalineUnit'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='daline Unit',
    install_requires=[
        'cx_Oracle',
        'google',
    ],
)

# python setup.py sdist bdist_wheel
# twine upload dist/*




import py_compile
import shutil

py_compile.compile('DalBaseUnit.py', r'..\DalineUnit\DalBaseUnit.pyc')
py_compile.compile('OracleOper.py', r'..\DalineUnit\OracleOper.pyc')
py_compile.compile('puwrapper_pb2.py', r'..\DalineUnit\puwrapper_pb2.pyc')
py_compile.compile('puwrapper_pb2_grpc.py', r'..\DalineUnit\puwrapper_pb2_grpc.pyc')
py_compile.compile('OracleOper.py', r'..\DalineUnit\OracleOper.pyc')
py_compile.compile('Template.py', r'..\DalineUnit\Template.pyc')
py_compile.compile('DalDecorate.py', r'..\DalineUnit\DalDecorate.pyc')
py_compile.compile('DalGenScript.py', r'..\DalineUnit\DalGenScript.pyc')
py_compile.compile('DesignFile.py', r'..\DalineUnit\DesignFile.pyc')
py_compile.compile('GenTestData.py', r'..\DalineUnit\GenTestData.pyc')
